<h3>Contents</h3>

<ul>
   <li>&nbsp; &nbsp;<a href="{{ pathto('install') }}"> Installation</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('pyshortcut_app') }}">  The `pyshortcut` program </a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('python') }}">  Use from Python</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('utilities') }}"> Utilitiy Functions</a></li>
</ul>

<h3>Install Pyshortcuts</h3>
<p>Current version: <b>{{ release }}</b></p>
<p>Install:   <tt>pip install pyshortcuts</tt></p>
<p>Develop:
  &nbsp; <a href="https://github.com/newville/pyshortcuts/">github.com</a>
  <p>

<hr>
